import "Simbo.Babble.DefaultFilters";

BabbleHelper = class();

local playerName = Turbine.Gameplay.LocalPlayer:GetInstance():GetName();
local playerNameLower = string.ToLower(playerName);
local meLower = string.ToLower(Strings["me"]);
local youLower = string.ToLower(Strings["you"]);

function BabbleHelper:Constructor(enabled, filterSet, nameList)
	self.Enabled = enabled;
	self.BlockedEmotes = {};
	self.ValidNames = {};

	self:SelectEmoteList(filterSet);
	self:SelectNameList(nameList);
end

function BabbleHelper:UpdateStatus(enabled, optionsPanel)
	self.Enabled = enabled;
	self:ShowStatus();
	
	if optionsPanel ~= nil then
		optionsPanel:EnableCheckbox(enabled);
	end
end

function BabbleHelper:ShowStatus()
	if self.Enabled then
		Turbine.Shell.WriteLine(Strings["babble_status_on"]);
	else
		Turbine.Shell.WriteLine(Strings["babble_status_off"]);
	end
end

function BabbleHelper:SelectEmoteList(emoteList)
	Settings.CurrentEmoteList = emoteList;
	
	self.PreparedFilters = { };
	if emoteList ~= nil then
		self.BlockedEmotes = Settings.EmoteLists[Settings.CurrentEmoteList];
	else
		self.BlockedEmotes = DefaultFilters[LANGUAGE];
	end

	local filterIndex = 1;
	local done = false;
	while not done do
		local stringIndex = tostring(filterIndex);
		local filterItem = self.BlockedEmotes[stringIndex];
		if filterItem ~= nil then
			local filter = filterItem.Text:ToLower();
			filter = filter:gsub("([%(%)%.%%%+%-%?%[%^%$%]])", "%%%1");
			filter = filter:gsub("*", ".+");
			filter = filter:gsub("_", "%%S+");
			filter = "^%S+ " .. filter;
			if not filter:match(".+[%.!]$") then
				filter = filter .. "%s*[%.!]?";
			end
			self.PreparedFilters[stringIndex] = filter .. "$";
			filterIndex = filterIndex + 1;
		else
			done = true;
		end
	end
end

function BabbleHelper:SelectNameList(nameList)
	Settings.CurrentNameList = nameList;

	if nameList ~= nil and nameList ~= "0" then
		self.ValidNames = Settings.NameLists[Settings.CurrentNameList];
	else
		self.ValidNames = { };
	end
end

function BabbleHelper:ListFilters()
	local filterIndex = 1;
	local done = false;
	while not done do
		local stringIndex = tostring(filterIndex);
		local filterItem = self.BlockedEmotes[stringIndex];
		if filterItem ~= nil then
			Turbine.Shell.WriteLine(filterItem.Text);
			filterIndex = filterIndex + 1;
		else
			done = true;
		end
	end
end

function BabbleHelper:ProcessEmote(message)
	if not self.Enabled then
		return;
	end
	
	if message:match(".+\n$") then
		message = message:sub(1, message:len() - 1);
	end
	
	local checkMessage = message:ToLower():gsub("\194\160", " ");
	
	local filterIndex = 1;
	local done = false;
	local matchesFilter = false;
	while not done and not matchesFilter do
		local stringIndex = tostring(filterIndex);
		local filter = self.PreparedFilters[stringIndex];
		if filter ~= nil then
			if checkMessage:match(filter) then
				local ignoreMatch = false;
				if self.BlockedEmotes[stringIndex].UnlessFrom and self.ValidNames ~= nil then
					local sourceName = checkMessage:match("^%S+");
					ignoreMatch = IsInList(self.ValidNames, sourceName);
				end
				if not ignoreMatch and self.BlockedEmotes[stringIndex].UnlessTo then
					for _, name in pairs(self.ValidNames) do
						name = string.ToLower(name);
						if name == meLower then
							name = playerNameLower;
						end
						ignoreMatch = checkMessage:match("^.+" .. name .. "%A") or checkMessage:match("^.+" .. name .. "$");
						if ignoreMatch then
							break;
						end
					end
				end
				matchesFilter = not ignoreMatch;
			end
			filterIndex = filterIndex + 1;
		else
			done = true;
		end
	end
	
	if not matchesFilter then
		Turbine.Shell.WriteLine("<rgb=#" .. Settings.EmoteColour .. ">" .. message .. "</rgb>");
	end
end

function IsInList(items, value)
	local found = false;
	local lowerValue = string.ToLower(value);
	for _, item in pairs(items) do
		local lowerItem = string.ToLower(item);
		if lowerItem == meLower or lowerItem == playerNameLower then
			if lowerValue == playerNameLower or lowerValue == youLower then
				found = true;
				break;
			end
		elseif lowerItem == lowerValue then
			found = true;
			break;
		end
	end
	
	return found;
end
