import "Simbo.Babble.WindowLotro";
import "Simbo.Babble.StyledList";
import "Simbo.Babble.EmoteList";
import "Simbo.Babble.ConfirmationWindow";

EmoteListWindow = class(WindowLotro);

local Constants =
{
	MinimumSize =
	{
		Width = 500,
		Height = 500
	},
	Padding =
	{
		Top = 50,
		Left = 20,
		Bottom = 20,
		Right = 20
	},
	LineSpacing = 8,
	ColumnSpacing = 8,
	Sizes =
	{
		LabelWidth = UIOptions["wide_name_list_label"] and 95 or 75,
		CheckBoxWidth = 80
	}
}

function EmoteListWindow:Constructor()
	WindowLotro.Constructor(self);
	
	self.ListId = nil;
	self.ListName = nil;
	self.Changed = false;
	
	self:SetText(Strings["emote_list"]);
	self:SetPosition(tonumber(Settings.EmoteListWindow.Position.Left) or 150, tonumber(Settings.EmoteListWindow.Position.Top) or 150);
	self:SetSize(tonumber(Settings.EmoteListWindow.Size.Width) or Constants.MinimumSize.Width, tonumber(Settings.EmoteListWindow.Size.Height) or Constants.MinimumSize.Height);
	
	self.AddWindow = ConfirmationWindow();
	self.AddWindow:SetZOrder(self:GetZOrder() + 10);
	self.AddWindow.Tag = nil;
	self.AddWindow.ButtonClick = function (sender,args)
		local text = args.Text:Trim();
		return self:SaveEmote(text, self.AddWindow.Tag);
	end

	local top = Constants.Padding.Top;
	local left = Constants.Padding.Left;
	
	self.UnlessFromLabel = CreateLabel(self, true);
	self.UnlessFromLabel:SetSize(Constants.Sizes.CheckBoxWidth, Globals.Sizes.Label.Height * 1.5);
	self.UnlessFromLabel:SetFont(Globals.Fonts.SmallLabel);
	self.UnlessFromLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	self.UnlessFromLabel:SetText(Strings["unless_from"]);
	self.UnlessFromLabel:SetTop(top);
	
	self.UnlessToLabel = CreateLabel(self, true);
	self.UnlessToLabel:SetSize(Constants.Sizes.CheckBoxWidth, Globals.Sizes.Label.Height * 1.5);
	self.UnlessToLabel:SetFont(Globals.Fonts.SmallLabel);
	self.UnlessToLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	self.UnlessToLabel:SetText(Strings["unless_to"]);
	self.UnlessToLabel:SetTop(top);
	
	top = top + self.UnlessFromLabel:GetHeight();
	
	self.EmoteLabel = CreateLabel(self);
	self.EmoteLabel:SetText(Strings["blocked_emote"]);
	self.EmoteLabel:SetPosition(left, top);
	
	self.UnlessFromCheckBox = CreateCheckBox(self)
	self.UnlessFromCheckBox:SetTop(top);
	self.UnlessFromCheckBox.IgnoreEvent = false;
	self.UnlessFromCheckBox.CheckedChanged = function(sender, args)
		self.Changed = true;
		if not self.UnlessFromCheckBox.IgnoreEvent then
			self.PatternList:SetAllCheckboxes(1, self.UnlessFromCheckBox:IsChecked());
		else
			self.UnlessFromCheckBox.IgnoreEvent = false;
		end
	end
	
	self.UnlessToCheckBox = CreateCheckBox(self)
	self.UnlessToCheckBox:SetTop(top);
	self.UnlessToCheckBox.IgnoreEvent = false;
	self.UnlessToCheckBox.CheckedChanged = function(sender, args)
		self.Changed = true;
		if not self.UnlessToCheckBox.IgnoreEvent then
			self.PatternList:SetAllCheckboxes(2, self.UnlessToCheckBox:IsChecked());
		else
			self.UnlessToCheckBox.IgnoreEvent = false;
		end
	end
	
	top = top + self.EmoteLabel:GetHeight();
	
	self.PatternList = EmoteList(self, 2);
	self.PatternList:SetPosition(left, top);
	self.PatternList:SetBorderColor(Globals.BorderColours.ListBox);
	self.PatternList:SetBackColor(Globals.BackgroundColours.ListBox);
	self.PatternList:SetForeColor(Globals.Colours.ListBox);
	self.PatternList:SetFont(Globals.Fonts.ListBox);
	self.PatternList:SetOpacity(0.95);
	self.PatternList.SelectedIndexChanged = function(sender, args)
		self.EditEmoteButton:SetEnabled(args.SelectedIndex > 0);
		self.RemoveEmoteButton:SetEnabled(args.SelectedIndex > 0);
		if self.AddWindow.Tag == "EDITEMOTE" then
			self.AddWindow:SetVisible(false);
			self.AddWindow.Tag = nil;
		end
	end
	self.PatternList.CheckBoxSelected = function(sender, args)
		self.Changed = true;
		
		local box = nil;
		if args.Index == 1 then
			box = self.UnlessFromCheckBox;
		else
			box = self.UnlessToCheckBox;
		end
		if args.AllSelected ~= box:IsChecked() then
			box.IgnoreEvent = true;
			box:SetChecked(args.AllSelected);
		end
	end
	
	self.AddEmoteButton = CreateButton(self, 0);
	self.AddEmoteButton:SetText(Strings["add"]);
	self.AddEmoteButton:SetLeft(left);
	self.AddEmoteButton.MouseClick = function(sender, args)
		self.AddWindow.Tag = "ADDEMOTE";
		self.AddWindow:ShowWindow(Strings["add_emote_title"], Strings["add_emote_message"], true, false);
	end
	
	self.EditEmoteButton = CreateButton(self, 0);
	self.EditEmoteButton:SetText(Strings["edit"]);
	self.EditEmoteButton:SetLeft(left + self.AddEmoteButton:GetWidth() + Constants.ColumnSpacing);
	self.EditEmoteButton.MouseClick = function(sender, args)
		local item = self.PatternList:GetSelectedItem();
		if item ~= nil then
			self.AddWindow.Tag = "EDITEMOTE";
			self.AddWindow:ShowWindow(Strings["edit_emote_title"], Strings["add_emote_message"], true, false, item.Text);
		end
	end
	
	self.RemoveEmoteButton = CreateButton(self, 0);
	self.RemoveEmoteButton:SetText(Strings["remove"]);
	self.RemoveEmoteButton:SetLeft(self.EditEmoteButton:GetLeft() + self.EditEmoteButton:GetWidth() + Constants.ColumnSpacing);
	self.RemoveEmoteButton.MouseClick = function(sender, args)
		self.Changed = true;
		self.PatternList:RemoveSelectedItem();
		self.EditEmoteButton:SetEnabled(false);
		self.RemoveEmoteButton:SetEnabled(false);
		if self.AddWindow.Tag == "EDITEMOTE" then
			self.AddWindow:SetVisible(false);
			self.AddWindow.Tag = nil;
		end
	end

	self.ClearListButton = CreateButton(self, UIOptions["wide_clear_list_button"] and 1 or 0);
	self.ClearListButton:SetText(Strings["clear_list"]);
	self.ClearListButton.MouseClick = function(sender, args)
		self.Changed = true;
		self.PatternList:ClearItems();
		self.EditEmoteButton:SetEnabled(false);
		self.RemoveEmoteButton:SetEnabled(false);
		if self.AddWindow.Tag == "EDITEMOTE" then
			self.AddWindow:SetVisible(false);
			self.AddWindow.Tag = nil;
		end
	end
	
	self.EmoteListNameLabel = CreateLabel(self);
	self.EmoteListNameLabel:SetText(Strings["list_name"]);
	self.EmoteListNameLabel:SetLeft(left);
	self.EmoteListNameLabel:SetWidth(Constants.Sizes.LabelWidth);
	
	self.EmoteListNameBox = CreateTextBox(self);
	self.EmoteListNameBox:SetLeft(self.EmoteListNameLabel:GetLeft() + self.EmoteListNameLabel:GetWidth() + Constants.ColumnSpacing);

	self.CancelButton = CreateButton(self);
	self.CancelButton:SetText(Strings["cancel"]);
	self.CancelButton.MouseClick = function(sender, args)
		self:Hide();
	end
	
	self.SaveButton = CreateButton(self);
	self.SaveButton:SetText(Strings["ok"]);
	self.SaveButton.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			local emoteList = self:SaveEmoteSettings();
			if emoteList ~= nil then
				args = { Id = self.ListId, List = emoteList, Changed = (self.Changed or (self.ListName ~= emoteList.Name)) };
				self.Saved(self, args);
				self:Hide();
			end
		end
	end
	
	self.VisibleChanged = function(sender, args)
		if not self:IsVisible() and not self.UIHidden then
			self.AddWindow:SetVisible(false);
		else
			self:CheckWindowPosition();
		end
	end
	
	self.Saved = function() end;
		
	self:LayoutControls();
	self:SetResizer(Constants.MinimumSize.Width, Constants.MinimumSize.Height);
end

function EmoteListWindow:LayoutControls()
	local listBoxWidth = self:GetWidth() - Constants.Padding.Left - Constants.Padding.Right;
	local listBoxHeight = self:GetHeight() - self.PatternList:GetTop() - Constants.Padding.Bottom - self.AddEmoteButton:GetHeight() - self.EmoteListNameLabel:GetHeight() - self.CancelButton:GetHeight() - Constants.LineSpacing * 5;

	self.PatternList:SetSize(listBoxWidth, listBoxHeight, Constants.Sizes.CheckBoxWidth);
	
	self.EmoteLabel:SetWidth(self.PatternList:GetLabelWidth());
	self.UnlessFromLabel:SetLeft(self.EmoteLabel:GetLeft() + self.EmoteLabel:GetWidth() - (Constants.Sizes.CheckBoxWidth - Globals.Sizes.CheckBoxOnly.Width) / 2);
	self.UnlessFromLabel:SetWidth(Constants.Sizes.CheckBoxWidth);
	self.UnlessFromCheckBox:SetLeft(self.EmoteLabel:GetLeft() + self.EmoteLabel:GetWidth());
	self.UnlessToLabel:SetLeft(self.UnlessFromLabel:GetLeft() + self.UnlessFromLabel:GetWidth());
	self.UnlessToLabel:SetWidth(Constants.Sizes.CheckBoxWidth);
	self.UnlessToCheckBox:SetLeft(self.UnlessFromCheckBox:GetLeft() + self.UnlessFromLabel:GetWidth());
	
	self.AddEmoteButton:SetTop(self.PatternList:GetTop() + self.PatternList:GetHeight() + Constants.LineSpacing);
	self.EditEmoteButton:SetTop(self.AddEmoteButton:GetTop());
	self.RemoveEmoteButton:SetTop(self.AddEmoteButton:GetTop());
	self.ClearListButton:SetPosition(self:GetWidth() - Constants.Padding.Right - self.ClearListButton:GetWidth(), self.AddEmoteButton:GetTop())

	self.EmoteListNameLabel:SetTop(self.AddEmoteButton:GetTop() + self.AddEmoteButton:GetHeight() + Constants.LineSpacing);
	self.EmoteListNameBox:SetTop(self.EmoteListNameLabel:GetTop());
	
	self.CancelButton:SetPosition((self:GetWidth() - (self.CancelButton:GetWidth() + Constants.ColumnSpacing + self.CancelButton:GetWidth())) / 2, self:GetHeight() - Constants.Padding.Bottom - self.CancelButton:GetHeight());
	self.SaveButton:SetPosition(self.CancelButton:GetLeft() + self.CancelButton:GetWidth() + Constants.ColumnSpacing, self.CancelButton:GetTop());
	
	self.EmoteListNameBox:SetWidth(self.SaveButton:GetLeft() + self.SaveButton:GetWidth() - self.EmoteListNameBox:GetLeft());
end

function EmoteListWindow:Show(listId, blockedEmotes, listName)
	self.ListId = listId;
	self.ListName = listName;
	self.Changed = false;

	self:PopulateSettings(blockedEmotes);
	self.EmoteListNameBox:SetText(listName);
	self.UnlessFromCheckBox:SetChecked(self.PatternList:GetItemCount() > 0 and self.PatternList:IsAllSelected(1));
	self.UnlessToCheckBox:SetChecked(self.PatternList:GetItemCount() > 0 and self.PatternList:IsAllSelected(2));
	
	self.AddWindow:SetVisible(false);
	self:SetVisible(true);
	self:Activate();
end

function EmoteListWindow:Hide()
	self.AddWindow:Close();
	self:Close();
end

function EmoteListWindow:PopulateSettings(blockedEmotes)
	self.PatternList:ClearItems();
	
	self.EditEmoteButton:SetEnabled(false);
	self.RemoveEmoteButton:SetEnabled(false);
	
	local itemIndex = 1;
	local done = false;
	while not done do
		local stringIndex = tostring(itemIndex);
		local filterItem = blockedEmotes[stringIndex];
		if filterItem ~= nil then
			self.PatternList:AddItem(filterItem.Text, { ["1"] = filterItem.UnlessFrom, ["2"] = filterItem.UnlessTo });
			itemIndex = itemIndex + 1;
		else
			done = true;
		end
	end
end

function EmoteListWindow:SaveEmote(text, mode)
	if string.len(text) > 0 then
		self.Changed = true;
		if mode == "ADDEMOTE" then
			local checkIndex = self.PatternList:FindInsertIndex(text);
			if checkIndex == nil then
				ShowBabbleMessage(Strings["emote_exists"], 2);
				return false;
			end
			self.PatternList:AddItem(text, { ["1"] = self.UnlessFromCheckBox:IsChecked(), ["2"] = self.UnlessToCheckBox:IsChecked() }, false);
		elseif mode == "EDITEMOTE" then
			local item = self.PatternList:GetSelectedItem();
			if item ~= nil and item.Text:Compare(text) ~= 0 then
				local checkIndex = self.PatternList:FindInsertIndex(text);
				if checkIndex == nil then
					ShowBabbleMessage(Strings["emote_exists"], 2);
					return false;
				end
				self.PatternList:RemoveSelectedItem();
				self.PatternList:AddItem(text, { ["1"] = item.CheckBoxes["1"], ["2"] = item.CheckBoxes["2"] }, true);
			end
		end
	end
	
	return true;
end

function EmoteListWindow:SavePositionSettings()
	Settings.EmoteListWindow.Position.Top = tostring(self:GetTop());
	Settings.EmoteListWindow.Position.Left = tostring(self:GetLeft());
	Settings.EmoteListWindow.Size.Width = tostring(self:GetWidth());
	Settings.EmoteListWindow.Size.Height = tostring(self:GetHeight());
end

function EmoteListWindow:SaveEmoteSettings()
	local setName = self.EmoteListNameBox:GetText():Trim();
	if setName == "" then
		ShowBabbleMessage(Strings["list_name_required"], 2);
		self.EmoteListNameBox:Focus();
		return nil;
	end
	
	local setDetails = { ["Name"] = setName };
	
	local emotes = self.PatternList:GetAllItems();
	local itemIndex = 1;
	local done = false;
	while not done do
		local stringIndex = tostring(itemIndex);
		local emote = emotes[stringIndex];
		if emote ~= nil then
			setDetails[stringIndex] =
			{
				Text = emote.Text,
				UnlessFrom = emote.CheckBoxes["1"],
				UnlessTo = emote.CheckBoxes["2"]
			}
			itemIndex = itemIndex + 1;
		else
			done = true;
		end
	end
	
	if itemIndex == 1 then
		ShowBabbleMessage(Strings["at_least_one_emote"], 2);
		return nil;
	end
	
	return setDetails;
end