import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.Gameplay"
import "Simbo.Babble.Localisation";
import "Simbo.Babble.Class"; -- Turbine library included so that there's no outside dependencies
import "Simbo.Babble.Events";
import "Simbo.Babble.StringFunctions";
import "Simbo.Babble.WindowFunctions";
import "Simbo.Babble.BabbleHelper";
import "Simbo.Babble.OptionsPanel";
import "Simbo.Babble.SettingsWindow";

Settings = 
{
	EmoteColour = "808080",
	EmoteListWindow = 
	{
		Position = {},
		Size = {}
	},
	NameListWindow = 
	{
		Position = {},
		Size = {}
	},
	SettingsWindow = 
	{
		Position = {},
		Size = {}
	},
	EmoteLists = nil,
	CurrentEmoteList = nil,
	NextEmoteListId = "1",
	NameLists = nil,
	CurrentNameList = nil,
	NextNameListId = "1"
}

Globals =
{
	Sizes =
	{
		Label = { Height = 20 },
		TextBox = { Width = 200, Height = 22 },
		Button = { Width = 90, Height = 24 },
		WideButton = { Width = 120, Height = 24 },
		ExtraWideButton = { Width = 160, Height = 24 },
		NarrowButton = { Width = 55, Height = 24 },
		CheckBoxOnly = { Width = 20, Height = 24 }
	},
	Fonts = 
	{
		Label = Turbine.UI.Lotro.Font.Verdana14,
		SmallLabel = Turbine.UI.Lotro.Font.Verdana12, 
		TextBox = Turbine.UI.Lotro.Font.Verdana14,
		ListBox = Turbine.UI.Lotro.Font.Verdana14
	},
	Colours =
	{
		Label = Turbine.UI.Color(1.0, 1.0, 1.0, 1.0),
		SecondaryLabel = Turbine.UI.Color(1.0, 0.5, 0.5, 0.5),
		TextBox = Turbine.UI.Color(1.0, 1.0, 1.0, 1.0),
		ListBox = Turbine.UI.Color(1.0, 1.0, 1.0, 1.0),
		Confirmation = Turbine.UI.Color((210 / 255), 1, 0),
		DefaultEmoteColour = Turbine.UI.Color((128 / 255), (128 / 255), (128 / 255)),
		Warning = Turbine.UI.Color(1.0, 1.0, 0.0, 0.0)
	},
	BackgroundColours =
	{
		TextBox = Turbine.UI.Color(1.0, 0, 0, 0),
		ListBox = Turbine.UI.Color(1.0, 0, 0, 0)
	},
	BorderColours =
	{
		ListBox = Turbine.UI.Color(1.0, 0.16, 0.19, 0.24)
	}
}

BabbleLoad = Turbine.PluginData.Load;
BabbleSave = Turbine.PluginData.Save;

Settings = BabbleLoad(Turbine.DataScope.Character, "BabbleSettings") or Settings;
if Settings.EmoteListWindow == nil then
	Settings.EmoteListWindow =
	{
		Position = {},
		Size = {}
	}
end

if Settings.NameListWindow == nil then
	Settings.NameListWindow = 
	{
		Position = {},
		Size = {}
	}
end

if Settings.EmoteLists == nil or Settings.EmoteLists["1"] == nil then
	Settings.NextEmoteListId = "2";
	Settings.EmoteLists =
	{
		["1"] = DefaultFilters[LANGUAGE];
	}
	Settings.EmoteLists["1"].Name = Strings["default_emote_set"];
end

if Settings.NameLists == nil then
	Settings.NextNameListId = "1";
	Settings.NameLists = {};
end

Settings.CurrentSet = nil;
Settings.FilterSets = nil;

BabbleHelper = BabbleHelper(true, Settings.CurrentEmoteList, Settings.CurrentNameList);

SettingsWindow = SettingsWindow();
SettingsWindow.Saved = function (sender, args)
	BabbleHelper:SelectEmoteList(Settings.CurrentEmoteList);
	BabbleHelper:SelectNameList(Settings.CurrentNameList);
	SaveBabbleSettings(false);
end
	
local optionsPanel = OptionsPanel();
optionsPanel:EnableCheckbox(BabbleHelper.Enabled);
plugin.GetOptionsPanel = function()
    return optionsPanel;
end

BabbleCommand = Turbine.ShellCommand();

function BabbleCommand:Execute(cmd, args)
	if args ~= nil then
		args = args:lower();
	end

	if args == Strings["cmd_babble_on"] then
		BabbleHelper:UpdateStatus(true, optionsPanel);
	elseif args == Strings["cmd_babble_off"] then
		BabbleHelper:UpdateStatus(false, optionsPanel);
	elseif args == Strings["cmd_babble_list"] then
		BabbleHelper:ListFilters(false);
	elseif args == Strings["cmd_babble_edit"] then
		SettingsWindow:Show();
	else
		BabbleCommand:GetHelp();
	end
end

function BabbleCommand:GetHelp()
	ShowBabbleMessage(Strings["babble_help_on"]);
	ShowBabbleMessage(Strings["babble_help_off"]);
	ShowBabbleMessage(Strings["babble_help_list"]);
	ShowBabbleMessage(Strings["babble_help_edit"]);
	ShowBabbleMessage("");
	BabbleHelper:ShowStatus();
end

function SaveBabbleSettings(showMessage)
	if showMessage then
		BabbleSave(Turbine.DataScope.Character, "BabbleSettings", Settings,
			function(result, message)
				if result then
					ShowBabbleMessage(Strings["settings_saved"], 1);
				else
					ShowBabbleMessage(Strings["settings_notsaved"] .. " " .. message, 2);
				end
			end);
	else
		BabbleSave(Turbine.DataScope.Character, "BabbleSettings", Settings);
	end
end

function ShowBabbleMessage(message, level)
	local display;
	if level == nil or level == 0 then
		display = "";
	elseif level == 1 then
		display = "<rgb=#00FF00>";
	elseif level == 2 then
		display = "<rgb=#FF0000>";
	end
	
	display = display .. message;
	
	if level ~= nil and level > 0 then
		display = display .. "</rgb>";
	end
	
	Turbine.Shell.WriteLine(display);
end

function Babble_ChatReceived(sender, args)
    if args.ChatType == Turbine.ChatType.Emote then
		BabbleHelper:ProcessEmote(args.Message);
    end
end

AddCallback(Turbine.Chat, "Received", Babble_ChatReceived);

Plugins["Babble"].Unload = function(sender, args)
	RemoveCallback(Turbine.Chat, "Received", Babble_ChatReceived);
	
	SettingsWindow:SavePositionSettings();
	SaveBabbleSettings(true);
end

Turbine.Shell.AddCommand(Strings["babble"], BabbleCommand);
ShowBabbleMessage(Strings["babble_version"]..Plugins["Babble"]:GetVersion().." - Simbo");
ShowBabbleMessage(Strings["babble_welcome"]);
BabbleHelper:ShowStatus();