import "Simbo.Babble.WindowLotro";
import "Simbo.Babble.DropdownList";
import "Simbo.Babble.ColourPicker";
import "Simbo.Babble.EmoteListWindow";
import "Simbo.Babble.NameListWindow";
import "Simbo.Babble.ConfirmationWindow";

SettingsWindow = class(WindowLotro);

local Constants =
{
	MinimumSize =
	{
		Width = 430,
		Height = 340
	},
	Padding =
	{
		Top = 50,
		Left = 20,
		Bottom = 20,
		Right = 20
	},
	Sizes =
	{
		LabelWidth = UIOptions["wide_settings_label"] and 115 or 100,
		Dropdown = 
		{
			Width = 200,
			Height = 20,
			Open = 120
		}
	},
	LineSpacing = 8,
	ColumnSpacing = 8
}

function SettingsWindow:Constructor()
	WindowLotro.Constructor(self);

	self.ChangedEmoteLists = {};
	self.ChangedNameLists = {};
	self.NextEmoteListId = tonumber(Settings.NextEmoteListId);
	self.NextNameListId = tonumber(Settings.NextNameListId);
	self.EmotesSaved = false;
	self.NamesSaved = false;
	self.ColourChanged = false;
	self.CopyMode = false;
	self.CancelPressed = false;
	self.SavePressed = false;
	
	self:SetText(Strings["settings"]);
	self:SetPosition(tonumber(Settings.SettingsWindow.Position.Left) or 100, tonumber(Settings.SettingsWindow.Position.Top) or 100);
	self:SetSize(tonumber(Settings.SettingsWindow.Size.Width) or Constants.MinimumSize.Width, tonumber(Settings.SettingsWindow.Size.Height) or Constants.MinimumSize.Height);
	if self:GetWidth() < Constants.MinimumSize.Width then
		self:SetWidth(Constants.MinimumSize.Width);
	end
	if self:GetHeight() < Constants.MinimumSize.Height then
		self:SetHeight(Constants.MinimumSize.Height);
	end
	
	self.EmoteListWindow = EmoteListWindow();
	self.EmoteListWindow.Saved = function (sender, args)
		self.EmotesSaved = self.EmotesSaved or args.Changed;
		self.ChangedEmoteLists[args.Id] = args.List;
		if args.Id == tostring(self.NextEmoteListId) then
			self.NextEmoteListId = self.NextEmoteListId + 1;
			self.EmoteListDropdown:AddItem(args.List.Name, args.Id, true);
			self:SetEmoteButtonStatus();
		elseif self.EmoteListDropdown:GetSelectedText() ~= args.List.Name then
			self.EmoteListDropdown:UpdateSelectedItem(args.List.Name, args.Id);
		end
		if args.Changed then
			self:SetSaveButtonsStatus(true);
		end
	end
	
	self.NameListWindow = NameListWindow();
	self.NameListWindow.Saved = function (sender, args)
		self.NamesSaved = self.NamesSaved or args.Changed;
		self.ChangedNameLists[args.Id] = args.List;
		if args.Id == tostring(self.NextNameListId) then
			self.NextNameListId = self.NextNameListId + 1;
			self.NameListDropdown:AddItem(args.List.Name, args.Id, true);
		elseif self.NameListDropdown:GetSelectedText() ~= args.List.Name then
			self.NameListDropdown:UpdateSelectedItem(args.List.Name, args.Id);
		end
		if args.Changed then
			self:SetSaveButtonsStatus(true);
		end
	end

	local top = Constants.Padding.Top;
	local left = Constants.Padding.Left;
	
	local emoteSetLabel = CreateLabel(self, false);
	emoteSetLabel:SetPosition(left, top);
	emoteSetLabel:SetWidth(Constants.Sizes.LabelWidth);
	emoteSetLabel:SetText(Strings["emote_list"]);
	
	left = left + emoteSetLabel:GetWidth() + Constants.ColumnSpacing;
	
	self.EmoteListDropdown = CreateDropdown(self, left, top, Constants.Sizes.Dropdown.Width, Constants.Sizes.Dropdown.Height, Constants.Sizes.Dropdown.Open);
	self.EmoteListDropdown:SetAutoSort(true);
	self.EmoteListDropdown.SelectedIndexChanged = function(sender, args)
		self.EmotesSaved = true;
		self.EmoteListWindow:Hide();
		self:SetSaveButtonsStatus(true);
	end
	
	top = top + emoteSetLabel:GetHeight() + Constants.LineSpacing;
	left = self.EmoteListDropdown:GetLeft();

	self.AddEmotesButton = CreateButton(self, 0);
	self.AddEmotesButton:SetPosition(left, top);
	self.AddEmotesButton:SetText(Strings["add"]);
	self.AddEmotesButton.Click = function(sender, args)
		if self.CopyMode then
			local listId = self.EmoteListDropdown:GetSelectedValue();
			local blockedEmotes = self.ChangedEmoteLists[listId] or Settings.EmoteLists[listId];
			self.EmoteListWindow:Show(tostring(self.NextEmoteListId), blockedEmotes);
		else
			self.EmoteListWindow:Show(tostring(self.NextEmoteListId), {}, "");
		end
	end
	
	left = left + self.AddEmotesButton:GetWidth() + Constants.ColumnSpacing;
	
	self.EditEmotesButton = CreateButton(self, 0);
	self.EditEmotesButton:SetPosition(left, top);
	self.EditEmotesButton:SetText(Strings["edit"])
	self.EditEmotesButton.Click = function(sender, args)
		local listId = self.EmoteListDropdown:GetSelectedValue();
		local blockedEmotes = self.ChangedEmoteLists[listId] or Settings.EmoteLists[listId];
		self.EmoteListWindow:Show(listId, blockedEmotes, blockedEmotes.Name);
	end
	
	left = left + self.EditEmotesButton:GetWidth() + Constants.ColumnSpacing;
	
	self.RemoveEmotesButton = CreateButton(self, 0);
	self.RemoveEmotesButton:SetPosition(left, top);
	self.RemoveEmotesButton:SetText(Strings["remove"])
	self.RemoveEmotesButton.Click = function(sender, args)
		local listId = self.EmoteListDropdown:GetSelectedValue();
		if listId ~= nil then
			self.EmotesSaved = true;
			self.ChangedEmoteLists[listId] = { Remove = true };
			self.EmoteListDropdown:RemoveSelectedItem();
			self:SetEmoteButtonStatus();
			self:SetSaveButtonsStatus(true);
		end
	end
	
	top = top + emoteSetLabel:GetHeight() + Constants.LineSpacing;
	left = Constants.Padding.Left;
	
	local nameListLabel = CreateLabel(self, false);
	nameListLabel:SetPosition(left, top);
	nameListLabel:SetWidth(Constants.Sizes.LabelWidth);
	nameListLabel:SetText(Strings["allowed_players"]);
	
	left = left + nameListLabel:GetWidth() + Constants.ColumnSpacing;
	
	self.NameListDropdown = CreateDropdown(self, left, top, Constants.Sizes.Dropdown.Width, Constants.Sizes.Dropdown.Height, Constants.Sizes.Dropdown.Open);
	self.NameListDropdown:SetAutoSort(true);
	self.NameListDropdown.SelectedIndexChanged = function(sender, args)
		self.NamesSaved = true;
		self.NameListWindow:Hide();
		self:SetNameButtonStatus();
		self:SetSaveButtonsStatus(true);
	end
	
	top = top + nameListLabel:GetHeight() + Constants.LineSpacing;
	left = self.NameListDropdown:GetLeft();
	
	self.AddNameListButton = CreateButton(self, 0);
	self.AddNameListButton:SetPosition(left, top);
	self.AddNameListButton:SetText(Strings["add"]);
	self.AddNameListButton.Click = function(sender, args)
		if self.CopyMode and self.NameListDropdown:GetSelectedValue() ~= "0" then
			local listId = self.NameListDropdown:GetSelectedValue();
			local names = self.ChangedNameLists[listId] or Settings.NameLists[listId] or {};
			self.NameListWindow:Show(tostring(self.NextNameListId), names);
		else
			self.NameListWindow:Show(tostring(self.NextNameListId), {});
		end
	end
	
	left = left + self.AddNameListButton:GetWidth() + Constants.ColumnSpacing;
	
	self.EditNameListButton = CreateButton(self, 0);
	self.EditNameListButton:SetPosition(left, top);
	self.EditNameListButton:SetText(Strings["edit"])
	if self.NextNameListId == 1 then
		self.EditNameListButton:SetEnabled(false);
	end
	self.EditNameListButton.Click = function(sender, args)
		local listId = self.NameListDropdown:GetSelectedValue();
		local names = self.ChangedNameLists[listId] or Settings.NameLists[listId] or {};
		self.NameListWindow:Show(listId, names, names.Name);
	end
	
	left = left + self.EditNameListButton:GetWidth() + Constants.ColumnSpacing;
	
	self.RemoveNameListButton = CreateButton(self, 0);
	self.RemoveNameListButton:SetPosition(left, top);
	self.RemoveNameListButton:SetText(Strings["remove"]);
	self.RemoveNameListButton:SetEnabled(self.NameListDropdown:GetSelectedValue() ~= "0");
	self.RemoveNameListButton.Click = function(sender, args)
		local listId = self.NameListDropdown:GetSelectedValue();
		if listId ~= nil then
			self.NamesSaved = true;
			self.ChangedNameLists[listId] = { Remove = true };
			self.NameListDropdown:RemoveSelectedItem();
			self:SetNameButtonStatus();
			self:SetSaveButtonsStatus(true);
		end
	end
	
	top = top + self.AddNameListButton:GetHeight() + Constants.LineSpacing;
	left = self.AddNameListButton:GetLeft();
	
	local copyLabel = CreateLabel(self, false);
	copyLabel:SetPosition(left, top);
	copyLabel:SetWidth(Constants.MinimumSize.Width - Constants.Padding.Left - Constants.Padding.Right);
	copyLabel:SetFont(Globals.Fonts.SmallLabel);
	copyLabel:SetText(Strings["copy_list"]);
	
	top = top + copyLabel:GetHeight() + Constants.LineSpacing;
	left = Constants.Padding.Left;
	
	local colourPickerLabel = CreateLabel(self, false);
	colourPickerLabel:SetPosition(left, top);
	colourPickerLabel:SetWidth(Constants.Sizes.LabelWidth);
	colourPickerLabel:SetText(Strings["emote_colour"]);
	
	left = left + colourPickerLabel:GetWidth() + Constants.ColumnSpacing;
	
	self.ColourPicker = ColourPicker(self);
	self.ColourPicker:SetPosition(left, top);
	self.ColourPicker.ColourSelected = function (sender, args)
		self.ColourChanged = true;
		self:SetSaveButtonsStatus(true);
	end
	
	top = top + self.ColourPicker:GetHeight() + Constants.LineSpacing * 2;
	left = Constants.Padding.Left;
	
	self.WarningLabel =  CreateLabel(self, true);
	self.WarningLabel:SetPosition(left, top);
	self.WarningLabel:SetHeight(self.WarningLabel:GetHeight() * 2);
	self.WarningLabel:SetForeColor(Globals.Colours.Warning);
	self.WarningLabel:SetText(Strings["changes_to_save"]);
	self.WarningLabel:SetVisible(false);
	
	local confirmWindow = ConfirmationWindow();
	confirmWindow:SetZOrder(self:GetZOrder() + 10);
	confirmWindow.ButtonClick = function (sender,args)
		self.CancelPressed = true;
		self:Hide();
	end
	
	self.CancelButton = CreateButton(self);
	self.CancelButton:SetText(Strings["cancel"]);
	self.CancelButton.MouseClick = function(sender, args)
		self:Hide();
	end
	
	self.ApplyButton = CreateButton(self);
	self.ApplyButton:SetText(Strings["apply"]);
	self.ApplyButton.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			self:SaveSettings(false);
		end
	end
	
	self.SaveButton = CreateButton(self);
	self.SaveButton:SetText(Strings["ok"]);
	self.SaveButton.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			self.SavePressed = true;
			self:SaveSettings(true);
		end
	end
	
	self.KeyDown = function (sender, args)
		if args.Shift then
			self:SetAddButtonStatus(true);
		end
		KeyDownActions(self, args);
	end
	
	self.KeyUp = function (sender, args)
		if args.Shift then
			self:SetAddButtonStatus(false);
		end
	end
	
	self.Closing = function(sender, args)
		if not (self.CancelPressed or self.SavePressed) and (self.EmotesSaved or self.NamesSaved or self.ColourChanged) then
			args.Cancel = true;
			confirmWindow:ShowWindow(Strings["settings"], Strings["changes_made"], false, true);
		else
			self.CancelPressed = false;
			self.SavePressed = false;
		end
	end
	
	self.VisibleChanged = function(sender, args)
		if not self:IsVisible() and not self.UIHidden then
			self.EmoteListWindow:SetVisible(false);
			self.NameListWindow:SetVisible(false);
		else
			self:CheckWindowPosition();
		end
	end

	self.Saved = function() end;
		
	self:LayoutControls();
	self:SetResizer(Constants.MinimumSize.Width, Constants.MinimumSize.Height);
end

function SettingsWindow:LayoutControls()
	local dropdownWidth = self:GetWidth() - Constants.Padding.Right - self.EmoteListDropdown:GetLeft();
	self.EmoteListDropdown:SetWidth(dropdownWidth);
	self.NameListDropdown:SetWidth(dropdownWidth);
	
	self.WarningLabel:SetWidth(self:GetWidth() - Constants.Padding.Right - Constants.Padding.Left);
	
	local buttonsWidth = self.CancelButton:GetWidth() + Constants.ColumnSpacing + self.ApplyButton:GetWidth() + Constants.ColumnSpacing + self.SaveButton:GetWidth();
	
	self.CancelButton:SetPosition((self:GetWidth() - buttonsWidth) / 2, self:GetHeight() - Constants.Padding.Bottom - self.CancelButton:GetHeight());
	self.ApplyButton:SetPosition(self.CancelButton:GetLeft() + self.CancelButton:GetWidth() + Constants.ColumnSpacing, self.CancelButton:GetTop());
	self.SaveButton:SetPosition(self.ApplyButton:GetLeft() + self.ApplyButton:GetWidth() + Constants.ColumnSpacing, self.CancelButton:GetTop());
end

function SettingsWindow:Show()
	self.CancelPressed = false;
	self.SavePressed = false;
	
	self:PopulateSettings();
	self:SetVisible(true);
	self:Activate();
end

function SettingsWindow:Hide()
	self:Close();
end

function SettingsWindow:PopulateSettings()
	self:ResetSettings();
	
	self.EmoteListDropdown:ClearItems();
	for key, value in pairs(Settings.EmoteLists) do
		self.EmoteListDropdown:AddItem(Settings.EmoteLists[key].Name, key);
	end
	self.EmoteListDropdown:SetSelectedValue(Settings.CurrentEmoteList or "1", false);
	self:SetEmoteButtonStatus();
	
	self.NameListDropdown:ClearItems();
	self.NameListDropdown:AddItem(Strings["no_name_list"], "0");
	for key, value in pairs(Settings.NameLists) do
		self.NameListDropdown:AddItem(Settings.NameLists[key].Name, key);
	end
	self.NameListDropdown:SetSelectedValue(Settings.CurrentNameList or "0", false);
	self:SetNameButtonStatus();
	
	self.ColourPicker:SetSelectedColourInHex(Settings.EmoteColour);
end

function SettingsWindow:ResetSettings()
	self.ChangedEmoteLists = {};
	self.ChangedNameLists = {};
	self.NextEmoteListId = tonumber(Settings.NextEmoteListId);
	self.NextNameListId = tonumber(Settings.NextNameListId);
	self.EmotesSaved = false;
	self.NamesSaved = false;
	self.ColourChanged = false;
	
	self.WarningLabel:SetVisible(false);
	self:SetSaveButtonsStatus(false);
end

function SettingsWindow:SetSaveButtonsStatus(enabled)
	self.ApplyButton:SetEnabled(enabled);
	self.SaveButton:SetEnabled(enabled);
	self.WarningLabel:SetVisible(enabled);
end

function SettingsWindow:SetEmoteButtonStatus()
	self.RemoveEmotesButton:SetEnabled(self.EmoteListDropdown:GetItemCount() > 1);
end

function SettingsWindow:SetNameButtonStatus()
	self.EditNameListButton:SetEnabled(self.NameListDropdown:GetSelectedValue() ~= "0");
	self.RemoveNameListButton:SetEnabled(self.NameListDropdown:GetSelectedValue() ~= "0");
end

function SettingsWindow:SetAddButtonStatus(copyMode)
	self.CopyMode = copyMode;
	
	local text = (copyMode and Strings["copy"]) or Strings["add"];
	
	self.AddEmotesButton:SetText(text);
	if self.NameListDropdown:GetSelectedValue() ~= "0" then
		self.AddNameListButton:SetText(text);
	end
end

function SettingsWindow:SaveSettings(closeWindow)
	if self.EmoteListWindow:IsVisible() or self.NameListWindow:IsVisible() then
		ShowBabbleMessage(Strings["close_windows"], 2);
		return;
	end
	
	if self.EmotesSaved then
		for key, value in pairs(self.ChangedEmoteLists) do
			if value.Remove then
				Settings.EmoteLists[key] = nil;
			else
				Settings.EmoteLists[key] = value;
			end
		end
	end
	Settings.CurrentEmoteList = self.EmoteListDropdown:GetSelectedValue();
	Settings.NextEmoteListId = tostring(self.NextEmoteListId);
	
	if self.NamesSaved then
		for key, value in pairs(self.ChangedNameLists) do
			if value.Remove then
				Settings.NameLists[key] = nil;
			else
				Settings.NameLists[key] = value;
			end
		end
	end
	Settings.CurrentNameList = self.NameListDropdown:GetSelectedValue();
	Settings.NextNameListId = tostring(self.NextNameListId);
	
	Settings.EmoteColour = self.ColourPicker:GetSelectedColourInHex();

	self.Saved(self, {});
	
	if closeWindow then
		self:Hide();
	else
		self:ResetSettings();
	end
end

function SettingsWindow:SavePositionSettings()
	Settings.SettingsWindow.Position.Top = tostring(self:GetTop());
	Settings.SettingsWindow.Position.Left = tostring(self:GetLeft());
	Settings.SettingsWindow.Size.Width = tostring(self:GetWidth());
	Settings.SettingsWindow.Size.Height = tostring(self:GetHeight());
	
	self.EmoteListWindow:SavePositionSettings();
	self.NameListWindow:SavePositionSettings();
end